/*
Copyright (c) 2014 DenshiKousakuSenka
 This software is released under the MIT License.
 http://opensource.org/licenses/mit-license.php
 */

#include "DKS_MPU6050.h"

DKS::MPU6050::MPU6050(const DKS::I2C::II2C *I2Cx, const bool PinAD0isHigh): i2c(I2Cx)
{
	if (PinAD0isHigh)
		slaveAddr = 0b11010010;	//0xD2
	else
		slaveAddr = 0b11010000;	//0xD0
}

bool DKS::MPU6050::TestConnection()const
{
	uint8_t buff;
	i2c->read(slaveAddr, 0x75, 1, &buff, 1);	//0x75 = WHO_AM_I
	return (buff == 0x68) ; //0b01101000;
}
uint8_t DKS::MPU6050::GetDeviceAddress()const
{
	return slaveAddr;
}

void DKS::MPU6050::Init(void)
{
	uint8_t buff;
	//DEVICE_RESET
	buff = 0x80; //0b10000000;
	i2c->write(slaveAddr, 0x6B, 1, &buff, 1);	//0x6B = PWR_MGMT_1   0x80=Device Reset
	for (int32_t i=0; i<100000; i++) asm("NOP");	// 6.3ms @ STM32F303VC (-O3 optimize most compile)

	do
	{
		i2c->read(slaveAddr, 0x6B, 1, &buff, 1);

	} while (buff & 0x80);	//check reset done

    //Wake UP
	buff = 0b00000001;	// Sleep=0, CYCLE=0, TEMP_DIS=0, MPU6050_CLOCK_PLL_XGYRO
	i2c->write(slaveAddr, 0x6B, 1, &buff, 1);

    setFullScaleGyroRange(GYRO_FS_250);
    setFullScaleAccelRange(ACCEL_FS_2);
}

void DKS::MPU6050::setFullScaleGyroRange(const FS_SEL scale)
{
	uint8_t buff(scale);
	i2c->write(slaveAddr, 0x1B, 1, &buff, 1);
	switch (scale)
	{
	case GYRO_FS_250:
		rateG = 250.0f / 32768.0f;
		break;
	case GYRO_FS_500:
		rateG = 500.0f / 32768.0f;
		break;
	case GYRO_FS_1000:
		rateG = 1000.0f / 32768.0f;
		break;
	case GYRO_FS_2000:
		rateG = 2000.0f / 32768.0f;
		break;
	}
}
void DKS::MPU6050::setFullScaleAccelRange(const AFS_SEL scale)
{
	uint8_t buff(scale);
	i2c->write(slaveAddr, 0x1C, 1, &buff, 1);
	switch (scale)
	{
	case ACCEL_FS_2:
		rateA = 2.0f / 32768.0f;
		break;
	case ACCEL_FS_4:
		rateA = 4.0f / 32768.0f;
		break;
	case ACCEL_FS_8:
		rateA = 8.0f / 32768.0f;
		break;
	case ACCEL_FS_16:
		rateA = 16.0f / 32768.0f;
		break;
	}
}
void DKS::MPU6050::setDigitalLowpassFilter(const LowPassFilter DLPF)const
{
	uint8_t buff;
	i2c->read(slaveAddr, 0x1A, 1, &buff, 1);
	buff = (buff & 0b11111000) + DLPF;
	i2c->write(slaveAddr, 0x1A, 1, &buff, 1);
}
void DKS::MPU6050::setSampleRateDivider(const uint8_t divider)const
{
	uint8_t buff(divider);
	i2c->write(slaveAddr, 0x19, 1, &buff, 1);
}
void DKS::MPU6050::getSampleRateDivider(uint8_t &divider)const
{
	i2c->read(slaveAddr, 0x19, 1, &divider, 1);
}

void DKS::MPU6050::GetData(int16_t Acc[3], int16_t &Temp, int16_t Gyro[3])const
{
	uint8_t buffer[14];
	i2c->read(slaveAddr, 0x3B, 1, buffer, 14);
	Acc[0]  = (((int16_t)buffer[0]) << 8) | buffer[1];
	Acc[1]  = (((int16_t)buffer[2]) << 8) | buffer[3];
	Acc[2]  = (((int16_t)buffer[4]) << 8) | buffer[5];
	Temp    = (((int16_t)buffer[6]) << 8) | buffer[7];
	Gyro[0] = (((int16_t)buffer[8]) << 8) | buffer[9];
	Gyro[1] = (((int16_t)buffer[10]) << 8) | buffer[11];
	Gyro[2] = (((int16_t)buffer[12]) << 8) | buffer[13];
}

void DKS::MPU6050::GetData(float Acc[3], float &Temp, float Gryo[3])const
{
	int16_t a[3] = {0};
	int16_t t(0);
	int16_t g[3] = {0};
	GetData(a, t, g);
	Acc[0] = a[0] * rateA;
	Acc[1] = a[1] * rateA;
	Acc[2] = a[2] * rateA;
	Temp = (t-521)*0.002777777777f + 36.53f;
	Gryo[0] = g[0] * rateG;
	Gryo[1] = g[1] * rateG;
	Gryo[2] = g[2] * rateG;
}
